@@ -4,7 +4,7 @@ from obitools.fasta import fastaNucIterator,formatFasta
 #from obitools.align.ssearch import ssearchIterator
 from obitools.utils.bioseq import uniqSequence,sortSequence
 
-from obitools.align import lenlcs
+from obitools.align import lenlcs,ALILEN
 
 from obitools.options.taxonomyfilter import addTaxonomyDBOptions,loadTaxonomyDatabase
 from obitools.ecobarcode.databases import referenceDBIterator
@@ -82,7 +82,7 @@ def addSearchOptions(optionManager):
                              default=None,
                              help='Sort output on input sequence tag')
     
-    optionManager.add_option('-r','--reserse',
+    optionManager.add_option('-r','--reverse',
                              action='store_true',dest='reverse',
                              default=False,
                              help='Sort in reverse order (should be used with -S)')
@@ -97,6 +97,11 @@ def addSearchOptions(optionManager):
                              default=False,
                              help='Switch to the new match algorithm')    
 
+    optionManager.add_option('-E','--errors',
+                             action='store',dest='error',
+                             default=0.0,
+                             help='Tolarated rate of wrong assignation')    
+
 
 def count(data):
     rep = {}
@@ -123,40 +128,38 @@ def lcsIterator(entries,db,options):
         maxid   = (None,0.0)
         minid   = options.minimum
         for d in db:
-            lcs,lali = lenlcs(seq,d,minid,normalized=True)
-#           lcs,lali = lenlcs(seq,d,minid,normalized=True, large=options.large)
+            lcs,lali = lenlcs(seq,d,minid,normalized=True,reference=ALILEN)
             if lcs > maxid[1]:
-#                try:
-#                    print "@@@@@@@@@@@>>>>>>",(maxid[0].id,maxid[1]),(d.id,lcs)
-#                except:
-#                    print "@@@@@@@@@@@>>>>>>",(None,maxid[1]),(d.id,lcs)
                 maxid = (d,lcs)
                 minid = maxid[1] ** options.shape
             results.append((d,lcs))
         minid = maxid[1] ** options.shape
         results = [x for x in results if x[1]>=minid]
-        yield seq,maxid,results
+        yield seq,([maxid[0]],maxid[1]),results
 
 
 def lcsIteratorSelf(entries,db,options):
     
     for seq in entries:
         results = []
-        maxid   = (None,0.0)
+        maxid   = ([],0.0)
         minid   = options.minimum
         for d in db:
-            lcs,lali = lenlcs(seq,d,minid,normalized=True)
+            lcs,lali = lenlcs(seq,d,minid,normalized=True,reference=ALILEN)
             if lcs > maxid[1]:
-                maxid = (d,lcs)
+                maxid = ([d],lcs)
                 minid = maxid[1]
+            elif lcs==maxid[1]:
+                maxid[0].append(d)
                 
-        if maxid[0] is not None:
-            results.append(maxid)
+        if maxid[0]:
+            results.extend([(s,maxid[1]) for s in maxid[0]])
             for d in db:
-                if d.id != maxid[0].id:
-                    lcs,lali = lenlcs(maxid[0],d,maxid[1],normalized=True)      
-                    if lcs >= maxid[1]:
-                        results.append((d,lcs))
+                for s in maxid[0]:
+                    if d.id != s.id:
+                        lcs,lali = lenlcs(s,d,maxid[1],normalized=True,reference=ALILEN)      
+                        if lcs >= maxid[1]:
+                            results.append((d,lcs))
                 
         yield seq,maxid,results
         
@@ -219,10 +222,15 @@ if __name__=='__main__':
 
         if best[0] is not None:
             taxlist = set(taxonlink[p[0].id] for p in match)
-            lca = taxonomy.lastCommonTaxon(*tuple(taxlist))
+            lca = taxonomy.betterCommonTaxon(options.error,*tuple(taxlist))
             scname = taxonomy.getScientificName(lca)
             rank = taxonomy.getRank(lca)
-      
+            if len(taxlist) < 15:
+                species_list = set(taxonomy.getSpecies(t) for t in taxlist)
+                species_list = [taxonomy.getScientificName(t) for t in species_list if t is not None]
+            else:
+                species_list = []
+                
             species = taxonomy.getSpecies(lca)
             if species is not None:
                 spname = taxonomy.getScientificName(species)
@@ -253,7 +261,7 @@ if __name__=='__main__':
                 
             worst = min(x[1] for x in match)
     
-            data =['ID',seq.id,best[0].id,best[1],worst,'%4.3f' %(best[1]**options.shape),seqcount,len(match),lca,scname,rank,order,orname,family,faname,genus,gnname,species,spname]
+            data =['ID',seq.id,best[0][0].id,best[1],worst,'%4.3f' %(best[1]**options.shape),seqcount,len(match),lca,scname,rank,order,orname,family,faname,genus,gnname,species,spname]
         else:
             data =['UK',seq.id,'--','--','--','--',seqcount,0,1,'root','no rank','-1','--','-1','--','-1','--','-1','--']
             
@@ -339,6 +347,7 @@ if __name__=='__main__':
                     seq['species_name']=data[18]
                 if options.explain is not None:
                     seq['explain']=dict((s[0].id,s[1]) for s in match)
+                seq['species_list']=species_list
             print formatFasta(seq)        
                 
         
