@@ -7,17 +7,19 @@ Created on 13 oct. 2009
 from obitools.format.sequence.embl import emblIterator
 from obitools.format.sequence.genbank import genbankIterator
 from obitools.format.sequence.fnaqual import fnaFastaIterator
-from obitools.format.sequence.fasta import fastaAAIterator,fastaNucIterator,fastaIterator
-from obitools.format.sequence.fastq import fastqIlluminaIterator,fastqSolexaIterator
-from obitools.fastq import fastqSangerIterator
+from obitools.format.sequence.fasta import fastaAAIterator, fastaNucIterator, fastFastaIterator
+from obitools.format.sequence.fastq import fastFastqIlluminaIterator,fastFastqSolexaIterator
+from obitools.fastq import fastFastqSangerIterator
 from obitools.fnaqual.quality import qualityIterator
 from obitools.fasta import formatFasta, rawFastaIterator,\
-    formatSAPFastaGenerator
+                           formatSAPFastaGenerator
 from obitools.fastq import formatFastq
 
 from obitools.ecopcr.sequence import EcoPCRDBSequenceWriter
 from obitools.ecopcr.options  import loadTaxonomyDatabase
 
+from cPickle import dump,load,UnpicklingError
+
 #from obitools.format._format import printOutput
 
 from array import array
@@ -26,14 +28,26 @@ import sys
 
 import re
 from obitools.ecopcr import EcoPCRFile
+from obitools.format.sequence import skipOnErrorIterator
+from obitools import BioSequence
+from obitools.utils import FakeFile
 
+def binarySequenceIterator(lineiterator):    
+            
+    f = FakeFile(lineiterator)
+            
+    try:
+        while(1):
+            try:
+                s = load(f)
+                yield s
+            except UnpicklingError:
+                pass
+    except EOFError:
+        raise StopIteration
+            
 
 def addInputFormatOption(optionManager):
-#    optionManager.add_option('--rank',
-#                             action="store_true", dest='addrank',
-#                             default=False,
-#                             help="add a rank attribute to the sequence "
-#                                  "indicating the sequence position in the input data")
     optionManager.add_option('--genbank',
                              action="store_const", dest="seqinformat",
                              default=None,
@@ -44,7 +58,12 @@ def addInputFormatOption(optionManager):
                              default=None,
                              const='embl',
                              help="input file is in embl format")
-
+    
+    optionManager.add_option('--skip-on-error',
+                             action="store_true", dest="skiperror",
+                             default=False,
+                             help="skip sequence entries with parse error")
+    
     optionManager.add_option('--fasta',
                              action="store_const", dest="seqinformat",
                              default=None,
@@ -106,6 +125,11 @@ def addInputFormatOption(optionManager):
         
 
 def addOutputFormatOption(optionManager):
+    optionManager.add_option('-B','--bin-output',
+                             action="store_const", dest="output",
+                             default=None,
+                             const=dump,
+                             help="output sequences in binary format")
     optionManager.add_option('--fastq-output',
                              action="store_const", dest="output",
                              default=None,
@@ -141,9 +165,6 @@ def addInOutputOption(optionManager):
     addOutputFormatOption(optionManager)
 
 
-
-
-
 def autoEntriesIterator(options):
     options.outputFormater=formatFasta
     options.outputFormat="fasta"
@@ -176,6 +197,7 @@ def autoEntriesIterator(options):
         options.outputFormater=formatFasta
         options.outputFormat="fasta"
         first = lineiterator.next()
+
         if first[0]==">":
             if options.withqualfile is not None:
                 qualfile=qualityIterator(options.withqualfile)
@@ -187,22 +209,27 @@ def autoEntriesIterator(options):
             elif options.moltype=='pep':
                 reader=fastaAAIterator
             else:
-                reader=fastaIterator
+                reader=fastFastaIterator
         elif first[0]=='@':
-            reader=fastqSangerIterator
+            reader=fastFastqSangerIterator
             options.outputFormater=formatFastq
             options.outputFormat="fastq"
         elif first[0:3]=='ID ':
             reader=emblIterator
         elif first[0:6]=='LOCUS ':
             reader=genbankIterator
+        elif first[0:8]=="#!Pickle":
+            reader=binarySequenceIterator
         elif first[0]=="#" or ecopcr_pattern.search(first):
             reader=EcoPCRFile 
         else:
             raise AssertionError,'file is not in fasta, fasta, embl, genbank or ecoPCR format'
         
-        input = reader(chain([first],lineiterator))
-        
+        if reader==binarySequenceIterator:
+            input = binarySequenceIterator(lineiterator)
+        else:
+            input = reader(chain([first],lineiterator))
+    
         return input
                 
     if options.seqinformat is None:
@@ -214,7 +241,7 @@ def autoEntriesIterator(options):
             elif options.moltype=='pep':
                 reader=fastaAAIterator
             else:
-                reader=fastaIterator
+                reader=fastFastaIterator
         elif options.seqinformat=='rawfasta':
             reader=annotatedIterator(rawFastaIterator)
         elif options.seqinformat=='genbank':
@@ -226,15 +253,15 @@ def autoEntriesIterator(options):
         elif options.seqinformat=='sanger':
             options.outputFormater=formatFastq
             options.outputFormat="fastq"
-            reader=fastqSangerIterator
+            reader=fastFastqSangerIterator
         elif options.seqinformat=='solexa':
             options.outputFormater=formatFastq
             options.outputFormat="fastq"
-            reader=fastqSolexaIterator
+            reader=fastFastqSolexaIterator
         elif options.seqinformat=='illumina':
             options.outputFormater=formatFastq
             options.outputFormat="fastq"
-            reader=fastqIlluminaIterator
+            reader=fastFastqIlluminaIterator
         elif options.seqinformat=='ecopcr':
             reader=EcoPCRFile
             
@@ -244,8 +271,9 @@ def autoEntriesIterator(options):
             options.outputFormater=formatFastq
             options.outputFormat="fastq"
             
-#    if options.addrank:
-#        reader = withRankIterator(reader)
+    if options.skiperror:
+        reader = skipOnErrorIterator(reader)
+
     return reader
 
 def sequenceWriterGenerator(options,output=sys.stdout):
@@ -270,9 +298,26 @@ def sequenceWriterGenerator(options,output=sys.stdout):
             except IOError:
                 sys.exit(0)
                 
+    class BinaryWriter:
+        def __init__(self,options,file=sys.stdout):
+            self._file=file
+            self._file.write("#!Pickle\n")
+        def put(self,seq):
+            try:
+                if isinstance(seq, BioSequence):
+                        dump(seq,self._file,protocol=2)
+                else:
+                    for s in seq:
+                        dump(s,self._file,protocol=2)
+            except IOError:
+                sys.exit(0)
+           
+                
     if options.ecopcroutput is not None:
         taxo = loadTaxonomyDatabase(options)
         writer=EcoPCRDBSequenceWriter(options.ecopcroutput,taxonomy=taxo)
+    elif options.output==dump:
+        writer=BinaryWriter(options,output)
     else:
         writer=SequenceWriter(options,output)
         
